@@ -1,18 +1,18 @@
-# 
-# Copyright (c) 2013, 2014, IBM Corp. All rights reserved. 
-# 		
-# This program is free software: you can redistribute it and/or modify 
-# it under the terms of the GNU General Public License as published by 
-# the Free Software Foundation, either version 3 of the License, or 
-# (at your option) any later version. 
 #
-# This program is distributed in the hope that it will be useful, 
-# but WITHOUT ANY WARRANTY; without even the implied warranty of 
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
-# GNU General Public License for more details. 
+# Copyright (c) 2013, 2014, 2018 IBM Corp. All rights reserved.
 #
-# You should have received a copy of the GNU General Public License 
-# along with this program. If not, see <http://www.gnu.org/licenses/>. 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 #
 
 idaConnect <- function (dsn,
@@ -55,15 +55,18 @@ idaConnect <- function (dsn,
     }
     idaCon <- odbcConnect(dsn, uid, pwd, believeNRows = FALSE)
     invisible(idaCon)
+    if (idaCon == -1 || !is(idaCon, "RODBC")) {
+        stop("Could not open a connection to the data base.");
+    }
     return(idaCon)
   }
 }
 
 #' isDSNString
-#' 
+#'
 #' Checks if string is a DSN connection string
 #'
-#' @param dsnString 
+#' @param dsnString
 #'
 #' @return true, if dsnString is DSN connection String
 
@@ -81,4 +84,3 @@ isDSNString <- function(dsnString) {
     FALSE
   }
 }
-
