@@ -1,5 +1,5 @@
 # 
-# Copyright (c) 2010, 2014, 2016, 2017 IBM Corp. All rights reserved. 
+# Copyright (c) 2010, 2014, 2016, 2018 IBM Corp. All rights reserved.
 # 		
 # This program is free software: you can redistribute it and/or modify 
 # it under the terms of the GNU General Public License as published by 
@@ -13,7 +13,8 @@
 #
 # You should have received a copy of the GNU General Public License 
 # along with this program. If not, see <http://www.gnu.org/licenses/>. 
-#
+# 
+# 
 
 utils::globalVariables(c("idaRGlobal"))
 
@@ -25,29 +26,30 @@ idaShowTables <- function(showAll=FALSE, matchStr=NULL, schema=NULL, accelerated
     stop("This function can only be applied to character values.")
   
   if(idaIsDb2z()) {
+  
     wherePart <- "WHERE (not s.CREATOR like 'SYS%') ";
     if(!showAll) {
-      currSchema <- idaGetCurrentSchema();	
+      currSchema <- idaGetCurrentSchema();
       wherePart <- paste("WHERE (s.CREATOR = '",currSchema,"') ",sep='');
-    } else if (!is.null(schema))  {	
+    } else if (!is.null(schema))  {
       wherePart <- paste("WHERE (s.CREATOR = '",schema,"') ",sep='');
     }
-    
+  
     if (accelerated) {
-      accelerator <- idaGetAccelerator(); 
+      accelerator <- idaGetAccelerator();
       if (accelerator == "") {
         stop('For the "accelerated=TRUE"" option the accelerator has to be set with the idaSetAccelerator function.')
       } else {
         wherePart <- paste( wherePart, "AND (a.ACCELERATORNAME = '", accelerator, "') ", sep="")
-      } 
+      }
     }
     if (!is.null(matchStr))
       wherePart <- paste(wherePart," AND s.NAME LIKE '%", paste(matchStr,collapse='%',sep=''), "%'", sep='')
     
-    query <- 	paste("SELECT distinct s.CREATOR as \"Schema\", s.NAME as \"Name\", s.OWNER as \"Owner\", s.TYPE as \"Type\", ", 
-                    "coalesce(a.ACCELERATORNAME, '') as \"Acceleratorname\", coalesce(a.ENABLE, ' ') as \"Enable\" ", 
-                    "from SYSIBM.SYSTABLES s left outer join SYSACCEL.SYSACCELERATEDTABLES a ",   
-                    "on s.CREATOR = a.CREATOR and s.NAME=a.NAME ", 
+    query <- 	paste("SELECT distinct s.CREATOR as \"Schema\", s.NAME as \"Name\", s.OWNER as \"Owner\", s.TYPE as \"Type\", ",
+                    "coalesce(a.ACCELERATORNAME, '') as \"Acceleratorname\", coalesce(a.ENABLE, ' ') as \"Enable\" ",
+                    "from SYSIBM.SYSTABLES s left outer join SYSACCEL.SYSACCELERATEDTABLES a ",
+                    "on s.CREATOR = a.CREATOR and s.NAME=a.NAME ",
                     wherePart,
                     "ORDER BY \"Schema\",\"Name\"")
     # idaQuery('SELECT distinct CREATOR as "Schema", NAME as "Name", OWNER as "Owner", TYPE as "Type" from SYSIBM.SYSTABLES ', wherePart ,'ORDER BY "Schema","Name"')
@@ -55,15 +57,15 @@ idaShowTables <- function(showAll=FALSE, matchStr=NULL, schema=NULL, accelerated
   } else {
     wherePart <- "WHERE (OWNERTYPE = 'U')";
     if(!showAll) {
-      currSchema <- idaGetCurrentSchema();	
+      currSchema <- idaGetCurrentSchema();
       wherePart <- paste(wherePart," AND (TABSCHEMA= '",currSchema,"') ",sep='');
     }
-    
+  
     if (!is.null(matchStr))
       wherePart <- paste(wherePart," AND TABNAME LIKE '%", paste(matchStr,collapse='%',sep=''), "%'", sep='')
-    
+  
     idaQuery('SELECT distinct TABSCHEMA as "Schema", TABNAME as "Name", OWNER as "Owner", TYPE as "Type" from SYSCAT.TABLES ', wherePart ,'ORDER BY "Schema","Name"')
-    
+
   }
 }
 
@@ -78,7 +80,7 @@ prepareCatalogLookup <- function(tableName) {
   if(idaIsDb2z()) {
     return(paste("NAME='",x$table,"'",ifelse(!is.null(x$schema),paste(" AND CREATOR = '",x$schema,"'",sep=''),''),sep=''));
   } else {
-    return(paste("TABNAME='",x$table,"'",ifelse(!is.null(x$schema),paste(" AND TABSCHEMA = '",x$schema,"'",sep=''),''),sep=''));			
+    return(paste("TABNAME='",x$table,"'",ifelse(!is.null(x$schema),paste(" AND TABSCHEMA = '",x$schema,"'",sep=''),''),sep=''));
   }
 }
 
@@ -175,7 +177,7 @@ idaGetValidTablespaceName <- function () {
     if (!idaExistTablespace(name))
       return(name)
   }
-  
+
   return(name)
 }
 
@@ -193,6 +195,17 @@ idaExistTable <- function (tableName) {
 }
 
 
+idaIsRegularTable <- function (tableName) {
+  idaCheckConnection();
+  cmp <- paste(prepareCatalogLookup(tableName), " AND TYPE = 'T'",sep='');
+  if(idaIsDb2z()) {
+    tableExists <- as.logical(as.integer(idaScalarQuery("SELECT CAST(COUNT(*) AS INTEGER) FROM SYSIBM.SYSTABLES WHERE ", cmp))>0);
+  } else {
+    tableExists <- as.logical(as.integer(idaScalarQuery("SELECT CAST(COUNT(*) AS INTEGER) FROM SYSCAT.TABLES WHERE ", cmp))>0);
+  }
+  return(tableExists);
+}
+
 idaIsView <- function (tableName) {
   idaCheckConnection();
   cmp <- paste(prepareCatalogLookup(tableName), " AND TYPE = 'V'",sep=''); 
@@ -209,7 +222,7 @@ idaListTableColumns <- function (tableName) {
   if(idaIsDb2z()) {
     x <- parseTableName(tableName);
     cmp <- paste("TBNAME='",x$table,"'",ifelse(!is.null(x$schema),paste(" AND TBCREATOR = '",x$schema,"'",sep=''),''),sep='')
-    return(as.vector(idaQuery("SELECT NAME FROM SYSIBM.SYSCOLUMNS WHERE ",cmp," ORDER BY COLNO ASC")[[1]]))  
+    return(as.vector(idaQuery("SELECT NAME FROM SYSIBM.SYSCOLUMNS WHERE ",cmp," ORDER BY COLNO ASC")[[1]]))
   } else {
     cmp <- prepareCatalogLookup(tableName)
     return(as.vector(idaQuery("SELECT COLNAME FROM SYSCAT.COLUMNS WHERE ",cmp," ORDER BY COLNO ASC")[[1]]))
@@ -230,6 +243,9 @@ idaExistColumnInTable <- function (columnName, tableName) {
   return(columnExists)
 }
 
+
+
+
 idaDeleteViewOrTable <- function(tableName) {
   if(idaIsView(tableName)) {
     idaDropView(tableName)
@@ -247,7 +263,7 @@ idaDeleteTable <- function(table) {
         accelerateTable(table, undo=TRUE)
       }
       idaQuery("DROP TABLE ", table)
-    }  
+    }
     invisible()
   }
 }
@@ -258,10 +274,9 @@ idaDropView <- function(v) {
 }
 
 idaDropDatabase <- function(dbname) {
-  if (idaExistDatabase(dbname)
-      && idaScalarQuery("select count(*) from SYSIBM.SYSTABLES where DBNAME = '", dbname, "'") == 0) {
+  if (idaExistDatabase(dbname) && idaScalarQuery("select count(*) from SYSIBM.SYSTABLES where DBNAME = '", dbname, "'") == 0) {
     idaQuery("DROP DATABASE ", dbname)
-  }  
+  }
 }
 
 idaDeleteTempTables <- function(pattern='DATA_FRAME_[0-9]+') {
@@ -291,8 +306,7 @@ idaDeleteTempTables <- function(pattern='DATA_FRAME_[0-9]+') {
       tempTab = tempTables[i,]
       idaDeleteViewOrTable(paste('"', sub('\\s+$', '', tempTab$Schema), '"."', tempTab$Name, '"', sep=''))
     } 
-  }
-  else if (choice=='b') {
+  } else if (choice=='b') {
     for (i in 1:nrow(tempTables)) {
       tempTab <- tempTables[i,]
       schema <- sub('\\s+$', '', tempTab$Schema)
@@ -301,14 +315,41 @@ idaDeleteTempTables <- function(pattern='DATA_FRAME_[0-9]+') {
       choice <- readline()
       if (choice=='e') {
         break
-      }
-      else if (choice=='y') {
+      } else if (choice=='y') {
         idaDeleteViewOrTable(paste('"', schema, '"."', tempTab$Name, '"', sep=''))
-      } 
-    } 
+      }
+    }
   }
 }
 
+
+idaRenameTable <- function(from=NULL, to=NULL, drop.to=T) {
+
+  if (is.null(from) || is.null(to)) {
+    # there is nothing to rename
+    stop("idaRenameTable: the from and to parameters have to be both non-null values.")
+  }
+  xTo <- parseTableName(to)
+  tryCatch({
+
+      odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = FALSE)
+      if(isTRUE(drop.to)) {
+        idaDeleteTable(to)
+      }
+      idaQuery("RENAME TABLE ", from , " TO ", xTo$table)
+      odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = TRUE);
+    }, error=function(e){
+      print(e);
+      odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = FALSE);
+      stop(paste("idaRenameTable(from=", from, ", to=", to, " ,drop.to=", drop.to, ") failed", sep=""))
+    }, finally={
+      odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = TRUE)
+    }
+  )
+}
+
+
+
 idaQuery <- function (..., as.is = TRUE, na.strings = "NA")  {
   
   idaCheckConnection();
@@ -316,10 +357,13 @@ idaQuery <- function (..., as.is = TRUE, na.strings = "NA")  {
   # for a DB2/z connection id the connection has been lost
   if (idaIsDb2z() & get("p_connectString", envir=idaRGlobal) != "") {
     conWorking=FALSE
-    try({conWorking <-odbcQuery(get("p_idaConnection", envir=idaRGlobal), "select count(*) from sysibm.sysdummy1")>0}, silent=TRUE)
+    try({
+      conWorking <-odbcQuery(get("p_idaConnection", envir=idaRGlobal), "select count(*) from sysibm.sysdummy1")>0
+      }, silent=TRUE
+    )
     if (!conWorking) {
       reconnect()
-    }	
+    }
   }
   
   query <- paste(..., sep = "", collapse = "")
@@ -329,7 +373,7 @@ idaQuery <- function (..., as.is = TRUE, na.strings = "NA")  {
   }
   
   result <- sqlQuery(get("p_idaConnection", envir=idaRGlobal), query, believeNRows = FALSE, 
-                     stringsAsFactors = FALSE, as.is = as.is, na.strings = na.strings)
+                      stringsAsFactors = FALSE, as.is = as.is, na.strings = na.strings)
   
   #we got an error message
   if (is.character(result) && length(result) > 0) {
@@ -366,23 +410,28 @@ callSP <- function (spname, retvalcolumn=NULL, ...) {
       view <- idaCreateView(value)
       tmp[length(tmp) + 1] = paste(name, "=", view, sep = "")
       views <- append(views, view)
-    }
-    else if (is.character(value) && length(grep(" ", value))) 
+    } else if (is.character(value) && length(grep(" ", value))) {
       tmp[length(tmp) + 1] = paste(name, "=\"", value, "\"", sep = "")
-    else if (length(value) > 1) 
+    } else if (length(value) > 1) {
       tmp[length(tmp) + 1] = paste(name, "=\"", paste(value, collapse = " "), "\"", sep = "")
-    else 
+    } else
       tmp[length(tmp) + 1] = paste(name, "=", value, sep = "")
   }
   
   if(idaIsDb2z()) {
-    acceleratorName = idaGetAccelerator(); 
-    try({idaQuery("DELETE FROM SESSION.INZA_MSG")});
-    retvalcolexpr <- ""; 
+    acceleratorName = idaGetAccelerator();
+    try({
+        idaQuery("DELETE FROM SESSION.INZA_MSG")
+      }
+     );
+    retvalcolexpr <- "";
     if (!is.null(retvalcolumn) && retvalcolumn != "")  {
       retvalcolexpr <- paste(", ", retvalcolumn, sep="")
-    } 
-    res <- try(idaQuery("CALL INZAR.", spname, "('", acceleratorName, "', '", paste(tmp, collapse = ","), "', 'SESSION.INZA_MSG(MESSAGE ", retvalcolexpr, ")')"), silent=T);
+    }
+    res <- try(
+        idaQuery("CALL INZAR.", spname, "('", acceleratorName, "', '", paste(tmp, collapse = ","), "', 'SESSION.INZA_MSG(MESSAGE ",
+                  retvalcolexpr, ")')"),
+        silent=T);
   } else {
     res <- try(idaQuery("CALL IDAX.", spname, "('", paste(tmp, collapse = ","), "')"), silent=T)
   }
@@ -393,20 +442,20 @@ callSP <- function (spname, retvalcolumn=NULL, ...) {
     if (regexpr("The operation was completed successfully.", message, fixed=TRUE) > 0)  {
       if (!is.null(retvalcolumn) && retvalcolumn != "")  {
         res <- try(idaQuery("SELECT ", retvalcolumn, " FROM SESSION.INZA_MSG"))[1,1]
-      } else if (!is.list(res) ) { # no result set has been returned 
+      } else if (!is.list(res) ) { # no result set has been returned
         res  <- matrix(c(0),1,1)
-      } 
+      }
     } else {
       startpos = regexpr("<message", message, fixed=TRUE) +9
       endpos = regexpr("</message>", message, fixed=TRUE) -1
       fullError <- gsub("&apos;", "'", gsub("&quot;", '"', substr(message, startpos, endpos)))
       #  fullError <- sprintf("The call of the stored procedure %s failed.", spname)
       stop(fullError)
-    } 
+    }
   } else if(inherits(res, "try-error")) {
     fullError <- idaScalarQuery("values idax.last_message")
     if(nchar(fullError)==0)
-      fullError <- res;    
+      fullError <- res;
     stop(fullError)
   }
   return(invisible(res))
@@ -418,35 +467,40 @@ accelerateTable <-function(tableName, undo=FALSE) {
     procName = "INZAR.ACCELERATE_TABLE"
     if (undo) {
       procName = "INZAR.ACCEL_REMOVE_TABLE"
-    }	
-    acceleratorName = idaGetAccelerator(); 
+    }
+    acceleratorName = idaGetAccelerator();
     x <- parseTableName(tableName);
     try({idaQuery("DELETE FROM SESSION.INZA_MSG")})
     idaQuery(paste("call ", procName, "('", acceleratorName, "', '", x$schema, "', '", x$table, "', 'SESSION.INZA_MSG(MESSAGE)')", sep=""))
     message <- sqlQuery(get("p_idaConnection",envir=idaRGlobal), "select MESSAGE from SESSION.INZA_MSG")$MESSAGE
-    if (regexpr("The operation was completed successfully.", message, fixed=TRUE) > 0)  {	
+    if (regexpr("The operation was completed successfully.", message, fixed=TRUE) > 0)  {
       return( matrix(c(0),1,1) )
     } else {
       startpos <- regexpr("<message", message, fixed=TRUE) +9
       endpos <- regexpr("</message>", message, fixed=TRUE) -1
       fullError <- gsub("&apos;", "'", gsub("&quot;", '"', substr(message, startpos, endpos)))
       stop(fullError)
-    }	
+    }
   }
 }	
 
+
 isAccelerated <- function(tableName) {
   if(idaIsDb2z()) {
     x <- parseTableName(tableName)
-    query <- paste("select count(*)from SYSIBM.SYSTABLES s left outer join SYSACCEL.SYSACCELERATEDTABLES a ",   
-                   "on s.CREATOR = a.CREATOR and s.NAME=a.NAME ", 
-                   "where s.CREATOR = '", x$schema, "'and s.NAME = '", x$table, "' and s.TYPE ='T'", sep="")
-    return(idaScalarQuery(query) > 0)						
+    query <- paste("select count(*)from SYSIBM.SYSTABLES s left outer join SYSACCEL.SYSACCELERATEDTABLES a ",
+            "on s.CREATOR = a.CREATOR and s.NAME=a.NAME ",
+            "where s.CREATOR = '", x$schema, "'and s.NAME = '", x$table, "' and s.TYPE ='T'", sep="")
+    return(idaScalarQuery(query) > 0)
   } else {
     return(FALSE)
   }
 }	
 
+
+
+
+
 ################ Table def ############################
 idaTableDef <- function(bdf,  collapse=TRUE) {
   if (!is.ida.data.frame(bdf))
@@ -460,7 +514,7 @@ idaTableDef <- function(bdf,  collapse=TRUE) {
   if(idaIsDb2z()) {
     x <- parseTableName(bdf@table);
     cmp <- paste("TBNAME='",x$table,"'",ifelse(!is.null(x$schema),paste(" AND TBCREATOR = '",x$schema,"'",sep=''),''),sep='')
-    attrs <- idaQuery("SELECT NAME AS COLNAME,trim(COLTYPE) AS TYPENAME FROM SYSIBM.SYSCOLUMNS WHERE ",cmp," ORDER BY COLNO ASC")  
+    attrs <- idaQuery("SELECT NAME AS COLNAME,trim(COLTYPE) AS TYPENAME FROM SYSIBM.SYSCOLUMNS WHERE ",cmp," ORDER BY COLNO ASC")
   } else {
     attrs <- idaQuery("SELECT COLNAME, TYPENAME FROM SYSCAT.COLUMNS WHERE ",prepareCatalogLookup(bdf@table), " ORDER BY COLNO")
   }
@@ -471,15 +525,17 @@ idaTableDef <- function(bdf,  collapse=TRUE) {
   
   idx <- tolower(attrs[,1]) %in% tolower(bdf@cols)
   
-  if (as.logical(collapse))
+  if (as.logical(collapse)) {
     return(paste(attrs[idx,1], attrs[idx,2], collapse=","))
-  else {
+  } else {
     res <- data.frame(name=attrs[idx,1], type=attrs[idx,2])
     res$valType <- ifelse(res$type %in% c('VARCHAR','CHARACTER','VARGRAPHIC','GRAPHIC','CLOB'),'CATEGORICAL',ifelse(res$type %in% c('SMALLINT', 'INTEGER','BIGINT','REAL','DOUBLE','FLOAT','DECIMAL','NUMERIC'),'NUMERIC','NONE'))
     return(res)
   }
 }
 
+
+
 idaMaterialize <- function(idf,tableName, asAOT=TRUE, dbname="") {
   
   v <- idaCreateView(idf)
@@ -488,34 +544,34 @@ idaMaterialize <- function(idf,tableName, asAOT=TRUE, dbname="") {
     stop("Table already exists, choose a different name.")	
   }	
   tryCatch({
-    odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = FALSE)
-    if(idaIsDb2z()) {
-      createDB2zTableAs(tableName, v, dbname=dbname, asAOT=asAOT)
+        
+      odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = FALSE)
+      if(idaIsDb2z()) {
+        createDB2zTableAs(tableName, v, dbname=dbname, asAOT=asAOT)
+        odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = TRUE);
+      } else {
+        idaQuery("CREATE TABLE ", tableName, " LIKE ", v," ORGANIZE BY ROW NOT LOGGED INITIALLY");
+      }
+        
+      idaQuery("INSERT INTO ", tableName, " SELECT * FROM ", v);
+      if(idaIsDb2z() && !asAOT) {
+        accelerateTable(tableName)
+      }
+
       odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = TRUE);
-    } else {
-      idaQuery("CREATE TABLE ", tableName, " LIKE ", v," ORGANIZE BY ROW NOT LOGGED INITIALLY");
+        
+    }, error=function(e){
+        print(e);
+        odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = FALSE);
+        if(idaIsDb2z()) idaDeleteTable(tableName)
+    }, finally={
+        odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = TRUE);idaDropView(v)
     }
-    
-    idaQuery("INSERT INTO ", tableName, " SELECT * FROM ", v);
-    
-    if(idaIsDb2z() && !asAOT) {
-      accelerateTable(tableName)
-    }
-    
-    odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = TRUE);
-  }, error=function(e) {
-    print(e);
-    odbcEndTran(get("p_idaConnection",envir=idaRGlobal), commit = FALSE); 
-    if(idaIsDb2z()) idaDeleteTable(tableName)
-  }, finally={
-    odbcSetAutoCommit(get("p_idaConnection",envir=idaRGlobal), autoCommit = TRUE);
-    idaDropView(v)
-  }
   );
 }
 
 idaAppend <- function(df, table) {
-  sqlSave(get("p_idaConnection", envir=idaRGlobal), dat=df, tablename = table, rownames=F, append=T);
+  sqlSave(get("p_idaConnection",envir=idaRGlobal), dat=df, tablename = table, rownames=F,append=T);
 }
 
 ##############################  Utilities ############################
@@ -526,9 +582,9 @@ removeQuotes <- function(x) {
   if (nchar(x) > 2 && substr(x, 1, 1) == '"' && substr(x, nchar(x), nchar(x)) == '"') {
     x <- substr(x, 2, nchar(x)-1)
     return(x)
-  }
-  else
+  } else {
     return (x)
+  }
 }
 
 quoteSQLIdentifier <- function(x) {
@@ -543,6 +599,7 @@ quoteSQLIdentifier <- function(x) {
   }
 }
 
+
 colName <- function(colDef) {
   if (!inherits(colDef, 'ida.col.def'))
     stop("This function can only be applied on ida.col.def object")
@@ -551,7 +608,7 @@ colName <- function(colDef) {
   tab <- colDef@table
   if (term %in% tab@cols)
     return(term)
-  
+    
   defCols <- tab@colDefs
   if (is.null(defCols) || length(defCols)==0)
     return(NULL)
@@ -574,108 +631,110 @@ idaLTrim <- function (x) sub("^\\s+", "", x)
 # res is the result of an DB2/z INZA procedure call
 # which is a data frame with columns SEQID and TABLE_DETAILS
 idaDataFrameFromResultSet <- function(res)  {
-  rows <- strsplit(res$TABLES_DETAILS, '\n') [[1]]
-  nrows = length(rows)
-  #first create an empty data frame
-  colnames = strsplit(rows[2], '|', fixed=TRUE )[[1]]
-  ncols = length(colnames)
-  colname = idaTrim(colnames[1])
-  trimmedColnames = c(colname)
-  df <- data.frame("X"=1:(nrows-3))
-  
-  for(i in 2:(ncols)){
-    colname = idaTrim(colnames[i])
-    trimmedColnames = c(trimmedColnames, colname)
-    df <- cbind(df, 1:(nrows-3))
-  }
-  colnames(df)<- trimmedColnames
-  
-  for (i in 4:(nrows)) {
+   rows <- strsplit(res$TABLES_DETAILS, '\n') [[1]]
+   nrows = length(rows)
+   #first create an empty data frame
+   colnames = strsplit(rows[2], '|', fixed=TRUE )[[1]]
+   ncols = length(colnames)
+   colname = idaTrim(colnames[1])
+   trimmedColnames = c(colname)
+   df <- data.frame("X"=1:(nrows-3))
+   
+   for(i in 2:(ncols)){
+      colname = idaTrim(colnames[i])
+      trimmedColnames = c(trimmedColnames, colname)
+      df <- cbind(df, 1:(nrows-3))
+   }
+   colnames(df)<- trimmedColnames
+   
+   for (i in 4:(nrows)) {
     rowvals = strsplit(rows[i], '|', fixed=TRUE )[[1]]
     for (j in 1:ncols) {
       df[i-3, j]<- idaRTrim(substring(rowvals[j],2))
     }
-  }
-  return(df)
+   }
+   return(df)
 }
 
 
 idaDataFrameFromResultStr.old <- function(resStr)  {
-  rows <- strsplit(resStr, '\n') [[1]]
-  nrows = length(rows)
-  #first create an empty data frame
-  colnames = strsplit(rows[2], '|', fixed=TRUE )[[1]]
-  ncols = length(colnames)
-  colname = idaTrim(colnames[1])
-  trimmedColnames = c(colname)
-  df <- data.frame("X"=1:(nrows-3))
-  
-  for(i in 2:(ncols)){
-    colname = idaTrim(colnames[i])
-    trimmedColnames = c(trimmedColnames, colname)
-    df <- cbind(df, 1:(nrows-3))
-  }
-  colnames(df)<- trimmedColnames
-  
-  if (nrows >= 4) {
-    for (i in 4:(nrows)) {
+   rows <- strsplit(resStr, '\n') [[1]]
+   nrows = length(rows)
+   #first create an empty data frame
+   colnames = strsplit(rows[2], '|', fixed=TRUE )[[1]]
+   ncols = length(colnames)
+   colname = idaTrim(colnames[1])
+   trimmedColnames = c(colname)
+   df <- data.frame("X"=1:(nrows-3))
+   
+   for(i in 2:(ncols)){
+      colname = idaTrim(colnames[i])
+      trimmedColnames = c(trimmedColnames, colname)
+      df <- cbind(df, 1:(nrows-3))
+   }
+   colnames(df)<- trimmedColnames
+   
+   if (nrows >= 4) {
+      for (i in 4:(nrows)) {
       rowvals = strsplit(rows[i], '|', fixed=TRUE )[[1]]
-      for (j in 1:ncols) {
-        df[i-3, j]<- idaRTrim(substring(rowvals[j],2))
+        for (j in 1:ncols) {
+          df[i-3, j]<- idaRTrim(substring(rowvals[j],2))
+        }
       }
-    }
-  } else {
-    # the result set is empty, but df has 2 rows, remove them
-    df <- df[-c(1,2),] 
-  }
-  return(df)
+   } else {
+      # the result set is empty, but df has 2 rows, remove them
+      df <- df[-c(1,2),]
+   }
+   return(df)
 }
 
 
 idaDataFrameFromResultStr <- function(resStr, rowSplitStr='\n', columnSplitStr = ' | ', headerRow=2, skippedDataRows=1, skippedRowChars=1)  {
-  skippedRows <- skippedDataRows + headerRow-1
-  rows <- strsplit(resStr, rowSplitStr, fixed=TRUE) [[1]]
-  nrows = length(rows)
-  #first create an empty data frame
-  colnames = strsplit(rows[headerRow], columnSplitStr, fixed=TRUE )[[1]]
-  ncols = length(colnames)
-  colname = idaTrim(colnames[1])
-  trimmedColnames = c(colname)
-  df <- data.frame("X"=1:(nrows-1-skippedRows))
-  
-  for(i in 2:(ncols)){
-    colname = idaTrim(colnames[i])
-    trimmedColnames = c(trimmedColnames, colname)
-    df <- cbind(df, 1:(nrows-1-skippedRows))
-  }
-  colnames(df)<- trimmedColnames
-  
-  if (nrows >= skippedRows+2) {
-    for (i in (skippedRows+2):(nrows)) {
+   skippedRows <- skippedDataRows + headerRow-1
+   rows <- strsplit(resStr, rowSplitStr, fixed=TRUE) [[1]]
+   nrows = length(rows)
+   #first create an empty data frame
+   colnames = strsplit(rows[headerRow], columnSplitStr, fixed=TRUE )[[1]]
+   ncols = length(colnames)
+   colname = idaTrim(colnames[1])
+   trimmedColnames = c(colname)
+   df <- data.frame("X"=1:(nrows-1-skippedRows))
+   
+   for(i in 2:(ncols)){
+      colname = idaTrim(colnames[i])
+      trimmedColnames = c(trimmedColnames, colname)
+      df <- cbind(df, 1:(nrows-1-skippedRows))
+   }
+   colnames(df)<- trimmedColnames
+   
+   if (nrows >= skippedRows+2) {
+      for (i in (skippedRows+2):(nrows)) {
       rowvals = strsplit(substr(rows[i], skippedRowChars+1, nchar(rows[i])), columnSplitStr, fixed=TRUE )[[1]]
-      for (j in 1:ncols) {
-        df[i-1-skippedRows, j]<- idaRTrim(rowvals[j])
+        for (j in 1:ncols) {
+          df[i-1-skippedRows, j]<- idaRTrim(rowvals[j])
+        }
       }
-    }
-  } else if (skippedRows > 0){
-    # the result set is empty, but df has the skipped rows, remove them
-    df <- df[-c(1,skippedRows),] 
-  }
-  
-  return(df)
+   } else if (skippedRows > 0){
+     # the result set is empty, but df has the skipped rows, remove them
+      df <- df[-c(1,skippedRows),]
+   }
+   
+   return(df)
 }
 
 
 createDB2zTableAs <- function(tableName, asTableView, asAOT=TRUE, dbname="") {
-  cols <- idaQuery("SELECT NAME,trim(COLTYPE) AS COLTYPE, LENGTH, SCALE FROM SYSIBM.SYSCOLUMNS WHERE TBNAME ='", asTableView,"' ORDER BY COLNO ASC")  
+  cols <- idaQuery("SELECT NAME,trim(COLTYPE) AS COLTYPE, LENGTH, SCALE FROM SYSIBM.SYSCOLUMNS WHERE TBNAME ='", asTableView,"' ORDER BY COLNO ASC")
   createTableStmt <- paste("CREATE TABLE ", tableName, " (", sep="")
   for(i in 1:nrow(cols)) {
-    colType <- cols[i,"COLTYPE"]
-    if (colType == "VARBIN") colType <- "VARBINARY"
-    else if (colType == "VARG") colType <- "VARGRAPHIC"
-    else if (colType == "TIMESTMP") colType <- "TIMESTAMP"
-    else if (colType == "TIMESTZ") colType <- "TIMESTAMP WITH TIME ZONE"
-    
+    colType0 <- cols[i,"COLTYPE"];
+    colType <- switch(colType0,
+                      "VARBIN"="VARBINARY",
+                      "VARG"="VARGRAPHIC",
+                      "TIMESTMP"="TIMESTAMP",
+                      "TIMESTZ"="TIMESTAMP WITH TIME ZONE",
+                      colType0);
+
     colDecl <- paste("\"", cols[i,"NAME"], "\" ", colType, sep="")
     if ( colType %in% c("CHAR", "VARCHAR", "LONGVAR", "BINARY", "VARBINARY", "GRAPHIC", "VARGRAPHIC", "DECIMAL", "LONGVARG", "DECFLOAT") ) {
       colDecl <- paste(colDecl, "(", cols[i,"LENGTH"], sep="")
@@ -689,17 +748,18 @@ createDB2zTableAs <- function(tableName, asTableView, asAOT=TRUE, dbname="") {
     }
     createTableStmt <- paste(createTableStmt, colDecl, sep="")
   }
-  createTableStmt <- paste(createTableStmt, ")", sep="")	
+  createTableStmt <- paste(createTableStmt, ")", sep="")
   if (asAOT) {
     createTableStmt <- paste(createTableStmt, " IN ACCELERATOR ", idaGetAccelerator(), sep="")
   } else if (dbname != "") {
     if (!idaExistDatabase(dbname)) {
-      idaQuery("CREATE DATABASE ", dbname, " CCSID ", idaGetAcceleratorEncoding())	
+      idaQuery("CREATE DATABASE ", dbname, " CCSID ", idaGetAcceleratorEncoding())
     }
     tbsp <- idaGetValidTablespaceName()
     idaQuery("CREATE TABLESPACE ", tbsp, " IN ", dbname, " CCSID ", idaGetAcceleratorEncoding())
     createTableStmt <- paste(createTableStmt, " IN ", dbname, ".", tbsp, sep = "")
-  } 
+  }
   createTableStmt <- paste(createTableStmt, " CCSID ", idaGetAcceleratorEncoding(), sep="")
   idaQuery(createTableStmt)
 }
+  
\ No newline at end of file
